import requests
from rcn_py import orcid
from rcn_py import scopus


# Get Scopus Author ID from ORCID
# This function is able to solve the problem of overlap between the RSD and Scopus databases
# And also return author's preferred name
# MYAPIKEY = "3d120b6ddb7d069272dfc2bc68af4028"
    

def get_scopus_result(authors, scopus_id_dict, preferred_name_dict, author_link_dict):
    """
        This function takes as inputs a list of authors, and three dictionaries which map an author's ORCID
        to their Scopus ID, preferred name, and author link, respectively. 

        For each author in the authors list, if they have an ORCID and it is not already in the Scopus ID dictionary,
        their Scopus information is fetched and added to all three dictionaries.

        Parameters:
        authors (list): A list of authors. Each author is expected to be a dictionary with an 'orcid' key.
        scopus_id_dict (dict): A dictionary that maps an author's ORCID to their Scopus ID.
        preferred_name_dict (dict): A dictionary that maps an author's ORCID to their preferred name.
        author_link_dict (dict): A dictionary that maps an author's ORCID to their author link.

        Returns:
        scopus_id_dict (dict): The updated Scopus ID dictionary.
        preferred_name_dict (dict): The updated preferred name dictionary.
        author_link_dict (dict): The updated author link dictionary.
    """
    # Iterate over each author in the authors list
    for author in authors:
        # If the author has an ORCID and it's not already in the Scopus ID dictionary
        if author['orcid'] and (author['orcid'] not in scopus_id_dict):
            # use the ORCID to get the Scopus ID, preferred name, and author link for the author
            scopus_id, preferred_name, author_link = scopus.get_scopus_info_from_orcid(author['orcid'])
            # Add the newly fetched information to their respective dictionaries
            scopus_id_dict[author['orcid']] = scopus_id
            preferred_name_dict[author['orcid']] = preferred_name
            author_link_dict[author['orcid']] = author_link

    # Return the updated dictionaries
    return scopus_id_dict, preferred_name_dict, author_link_dict

def request_rsd_data():
    """
    This function sends GET requests to the RSD API to retrieve data about
    projects, team members (authors), software and contributors. It then processes this data by adding full
    names and fetching missing ORCIDs, and returns it.

    Returns:
    projects (list): List of project data fetched from the API.
    authors_proj (list): List of author data fetched from the API.
    software (list): List of software data fetched from the API.
    contributor_soft (list): List of contributor data fetched from the API.
    """
    # Send GET requests to the RSD API and store the responses
    response_project = requests.get(
        "https://research-software-directory.org/api/v1/project?select=id,title,created_at,description"
        )
    response_author = requests.get(
        "https://research-software-directory.org/api/v1/team_member?select=id,project,orcid,given_names,family_names, affiliation"
        )
    response_software = requests.get(
        "https://research-software-directory.org/api/v1/software?select=id,concept_doi,brand_name,created_at,description"
        )
    response_contributor = requests.get(
        "https://research-software-directory.org/api/v1/contributor?select=id,software,orcid,given_names,family_names,affiliation"
        )
        
    # Convert the responses to JSON format
    projects = response_project.json()
    authors_proj = response_author.json()
    software = response_software.json()
    contributor_soft = response_contributor.json()

    # For each author, create a 'name' field and, if their ORCID is missing, generate one based on their name
    for a in authors_proj:
        a['name'] = a['given_names'] + ' ' + a['family_names']
        if a['orcid'] == None:
            a['orcid'] = orcid.name_to_orcid_id(a['name'])
    # For each contributor, create a 'name' field and, if their ORCID is missing, generate one based on their name
    for b in contributor_soft:
        b['name'] = b['given_names'] + ' ' + b['family_names']
        if b['orcid'] == None:
            b['orcid'] = orcid.name_to_orcid_id(b['name'])
    
    return projects, authors_proj, software, contributor_soft

def create_person_nodes(tx, authors, scopus_id_dict, preferred_name_dict, author_link_dict):
    """
    This function is used to create person nodes in a Neo4j database. It goes through each author, fetches their
    Scopus ID, preferred name and Scopus link, and then creates a new node in the database for them, or updates 
    the existing one if it already exists.

    Parameters:
    tx (neo4j.Session): A Neo4j session for running Cypher commands.
    authors (list): A list of authors.
    scopus_id_dict (dict): A dictionary that maps an author's ORCID to their Scopus ID.
    preferred_name_dict (dict): A dictionary that maps an author's ORCID to their preferred name.
    author_link_dict (dict): A dictionary that maps an author's ORCID to their author link.
    """
    # Iterate over each author in the list
    for author in authors:
        if author['orcid']:

            # If the author's ORCID is in the Scopus ID dictionary, get their Scopus ID, preferred name, and Scopus link
            # Otherwise, fetch this information using the get_scopus_info_from_orcid function
            if author['orcid'] in scopus_id_dict:
                author_scopus_id = scopus_id_dict[author['orcid']]
                preferred_name = preferred_name_dict[author['orcid']] 
                scopus_link = author_link_dict[author['orcid']]
            else:
                author_scopus_id, preferred_name, scopus_link = scopus.get_scopus_info_from_orcid(author['orcid'])
        
            # If no preferred name is returned, use the author's name as their preferred name
            if len(preferred_name) == 0:
                preferred_name = author['name']

            # If the author has no affiliation, create or update a node for them in the database using their ORCID and/or Scopus ID
            # Otherwise, also set their affiliation
            if author['affiliation'] is None:
                if len(author_scopus_id) == 0:
                    tx.run("""
                            MERGE (p:Person {orcid: $orcid})
                            SET p.scopus_link = $link,
                                p.name= $name
                            """,
                            orcid = author['orcid'],
                            link = scopus_link,
                            name = preferred_name
                            )
                else:
                    tx.run("""
                            MERGE (p:Person {scopus_id: $scopus_id})
                            SET p.orcid= $orcid, 
                                p.scopus_link = $link,
                                p.name= $name
                            """,
                            scopus_id = author_scopus_id, 
                            orcid = author['orcid'],
                            link = scopus_link,
                            name = preferred_name
                            )
            else:
                if len(author_scopus_id) == 0:
                    tx.run("""
                        MERGE (p:Person {orcid: $orcid})
                        SET p.scopus_link = $link,
                            p.name= $name,
                            p.affiliation= $affiliation
                        """,
                        orcid = author['orcid'],
                        link = scopus_link,
                        name = preferred_name,
                        affiliation = author['affiliation']
                        )
                else:
                    tx.run("""
                        MERGE (p:Person {scopus_id: $scopus_id})
                        SET p.orcid= $orcid, 
                            p.scopus_link = $link,
                            p.name= $name,
                            p.affiliation= $affiliation
                        """,
                        scopus_id = author_scopus_id,
                        orcid = author['orcid'],
                        link = scopus_link,
                        name = preferred_name,
                        affiliation = author['affiliation']
                        )
    # After processing all authors, print
    print("Author nodes added")
        

def create_project_nodes(tx, projects):
    """
    This function is used to create project nodes in a Neo4j database. It goes through each project, 
    fetches its title, year of creation, and description, and then creates a new node in the database 
    for each project.

    Parameters:
    tx (neo4j.Session): A Neo4j session for running Cypher commands.
    projects (list): A list of projects, where each project is a dictionary with 'id', 'title', 'created_at', 
                     and 'description' keys.
    """
    # Iterate over each project in the list
    for project in projects:
        # Create a new project node in the database with the project's ID, title, creation year, and description
        tx.run("""
            MERGE (p:Project {project_id: $proj_id})
            SET p.title = $title,
                p.year = $year,
                p.description = $description
            """,
            proj_id = project['id'],
            title = project['title'],
            year = project['created_at'][0:4],
            description = project['description']
            )
    print("Project nodes added")

def create_software_nodes(tx, softwares):
    """
    This function is used to create software nodes in a Neo4j database. It goes through each software, 
    fetches its DOI, brand name, year of creation, and description, and then creates a new node in the database 
    for each software.

    Parameters:
    tx (neo4j.Session): A Neo4j session for running Cypher commands.
    softwares (list): A list of softwares, where each software is a dictionary with 'id', 'concept_doi', 
                      'brand_name', 'created_at', and 'description' keys.
    """
    # Iterate over each software in the list
    for software in softwares:

        # Create a new software node in the database with the software's ID, DOI, brand name, creation year, 
        # and description
        tx.run("""
            MERGE (s:Software {software_id: $software_id})
            SET s.doi = $doi,
                s.title = $brand_name,
                s.year = $year,
                s.description = $description
            """,
            software_id = software['id'],
            doi = software['concept_doi'],
            brand_name = software['brand_name'],
            year = software['created_at'][0:4],
            description = software['description']
            )
    print("Software nodes added")

def create_author_project_edge(tx, authors, scopus_id_dict, preferred_name_dict):
    """
    This function creates relationships between authors and projects in the Neo4j graph database.
    For each author, it checks if their ORCID id exists in the Scopus id dictionary.

    Parameters:
    tx (neo4j.Session): A Neo4j session for running Cypher commands.
    authors (list): A list of authors, each represented as a dictionary.
    scopus_id_dict (dict): A dictionary mapping authors' ORCID ids to their Scopus ids.
    preferred_name_dict (dict): A dictionary mapping authors' ORCID ids to their preferred names.
    """
    for author in authors:
        # Check if the author's ORCID is present in the Scopus id dictionary.
        # If it is present, get the Scopus id and the preferred name for the author.
        if author['orcid'] in scopus_id_dict:
            author_scopus_id = scopus_id_dict[author['orcid']]
            preferred_name = preferred_name_dict[author['orcid']] 
        # If not present, get the Scopus id, preferred name and Scopus link using the author's ORCID.
        else:
            author_scopus_id, preferred_name, scopus_link = scopus.get_scopus_info_from_orcid(author['orcid'])

        # If the preferred name is empty, use the author's name.
        if len(preferred_name) == 0:
            preferred_name = author['name']

        # Execute a Cypher command to create a relationship between the author and the project
        # If the author Scopus id is empty, match the author node using the ORCID id
        # If not, match using the Scopus id.
        if len(author_scopus_id) == 0:
            tx.run("""
                MATCH 
                    (n:Person {orcid: $orcid}),
                    (p:Project {project_id: $proj_id})
                MERGE (n)-[r:IS_AUTHOR_OF]->(p)
                ON CREATE SET 
                    r.author_name = $author_name,
                    r.title = p.title,
                    r.year = p.year
                """,
                orcid = author['orcid'],
                proj_id = author['project'],
                author_name = preferred_name
            )
        else:
            tx.run("""
                MATCH 
                    (n:Person {scopus_id: $scopus_id}),
                    (p:Project {project_id: $proj_id})
                MERGE (n)-[r:IS_AUTHOR_OF]->(p)
                ON CREATE SET 
                    r.author_name = $author_name,
                    r.title = p.title,
                    r.year = p.year
                """,
                scopus_id = author_scopus_id,
                proj_id = author['project'],
                author_name = preferred_name
            )
    print("Author-Project relationship added")


def create_author_software_edge(tx, contributors, scopus_id_dict, preferred_name_dict):
    """
    This function is used to create edges between authors (contributors) and software in a Neo4j database. 
    It iterates over each author (contributor) and links them to their respective software.

    Parameters:
    tx (neo4j.Session): A Neo4j session for running Cypher commands.
    contributors (list): A list of authors (contributors), each represented as a dictionary.
    scopus_id_dict (dict): A dictionary mapping authors' (contributors') ORCID ids to their Scopus ids.
    preferred_name_dict (dict): A dictionary mapping authors' (contributors') ORCID ids to their preferred names.
    """
    for contributor in contributors:
        # Check if the contributor's ORCID is in the Scopus id dictionary.
        # If it is, fetch the corresponding Scopus id and preferred name.
        if contributor['orcid'] in scopus_id_dict:
            contributor_scopus_id = scopus_id_dict[contributor['orcid']]
            preferred_name = preferred_name_dict[contributor['orcid']] 
        # If the ORCID is not in the dictionary, get the Scopus id, preferred name, and Scopus link using the ORCID.
        else:
            contributor_scopus_id, preferred_name, scopus_link = scopus.get_scopus_info_from_orcid(contributor['orcid'])
              
        # If the preferred name is empty, use the contributor's name.
        if len(preferred_name) == 0:
            preferred_name = contributor['name']

        # If the Scopus id is empty, match the contributor node using the ORCID id. 
        # Else, match using the Scopus id.
        # In either case, create a relationship between the contributor and the software.
        if len(contributor_scopus_id) == 0:
            tx.run("""
                MATCH 
                    (n:Person {orcid: $orcid}),
                    (s:Software {software_id: $software_id})
                MERGE (n)-[r:IS_AUTHOR_OF]->(s)
                ON CREATE SET 
                    r.author_name = $author_name,
                    r.title = s.title,
                    r.year = s.year
                """,
                orcid = contributor['orcid'],
                software_id = contributor['software'],
                author_name = preferred_name
            )
        else:
            tx.run("""
                MATCH 
                    (n:Person {scopus_id: $scopus_id}),
                    (s:Software {software_id: $software_id})
                MERGE (n)-[r:IS_AUTHOR_OF]->(s)
                ON CREATE SET 
                    r.author_name = $author_name,
                    r.title = s.title,
                    r.year = s.year
                """,
                scopus_id = contributor_scopus_id,
                software_id = contributor['software'],
                author_name = preferred_name
            )
    print("Contributor-Software relationship added")


import networkx as nx
from pyvis.network import Network

from rcn_py import data_storage


# pyvis
def pyvis_network_by_datafile(orcid_id):
    node_data = data_storage.assign_group_node(orcid_id)
    edge_data, link = data_storage.get_links_from_coauthor_rel(orcid_id)
    all_node_in_link = (
        edge_data["source"].values.tolist() + edge_data["target"].values.tolist()
    )

    # getting weights of links
    weights = []
    for i in link:
        weights.append(link.count(i))
    edge_data["weight"] = weights

    # getting a group id
    groups = node_data.groupby("group")["orcid"].apply(list).reset_index()
    groups["group"] = groups.index

    # finding group id for each node from groups dataframe
    nodes = node_data.merge(groups, how="left", on=["group"])
    nodes["title"] = node_data[["name"]]
    nodes["label"] = node_data[["name"]]
    nodes = nodes.drop("orcid_y", axis=1).set_index("orcid_x")

    colors = [
        "#1f78b4",
        "#e08a34",
        "#643b8a",
        "#3d7a36",
        "#a34b43",
        "#47a68b",
        "blue",
        "red",
        "green",
        "black",
    ]
    node_color = [colors[n] for n in nodes["group"]]
    nodes["color"] = node_color
    # the size of the node could depend on the value, just a test
    nodes["value"] = [all_node_in_link.count(n) for n in node_data["orcid"]]

    # Pyvis network
    N = Network(height=800, width="100%", bgcolor="#222222", font_color="white")
    N.toggle_hide_edges_on_drag(False)
    N.barnes_hut()

    # collecting node attributes for network x
    # node_attrs = nodes.to_dict("index")

    # creating a network x graph from dataframes
    graph = nx.from_pandas_edgelist(edge_data, edge_attr=True)

    d = dict(graph.degree)
    d.update((x, 100 * y) for x, y in d.items())
    nodes["value"] = d.values()

    # collecting node attributes for network x
    node_attrs = nodes.to_dict("index")
    nx.set_node_attributes(graph, node_attrs)

    d = dict(graph.degree)
    d.update((x, 100 * y) for x, y in d.items())
    nx.set_node_attributes(graph, d, "size")

    N.from_nx(graph)
    N.show("test_datafile.html")


# gephi
def gephi_network_by_datafile(orcid_id):
    node_data = data_storage.assign_group_node(orcid_id)
    edge_data, link = data_storage.get_links_from_csv(orcid_id)
    weights = []

    all_node_in_link = (
        edge_data["source"].values.tolist() + edge_data["target"].values.tolist()
    )
    for i in link:
        weights.append(link.count(i))
    edge_data["weight"] = weights

    # getting a group id
    groups = node_data.groupby("group")["orcid"].apply(list).reset_index()
    groups["group"] = groups.index

    # finding group id for each node from groups dataframe
    nodes = node_data.merge(groups, how="left", on=["group"])
    nodes["title"] = node_data[["name"]]
    nodes["label"] = node_data[["name"]]
    nodes = nodes.drop("orcid_y", axis=1).set_index("orcid_x")

    # setting color for each group
    colors = [
        "#1f78b4",
        "#e08a34",
        "#643b8a",
        "#3d7a36",
        "#a34b43",
        "#47a68b",
        "blue",
        "red",
        "green",
        "black",
    ]
    node_color = [colors[n] for n in nodes["group"]]
    nodes["color"] = node_color

    # the size of the node could depend on the value, just a test
    nodes["value"] = [all_node_in_link.count(n) for n in node_data["orcid"]]

    # collecting node attributes for network x
    node_attrs = nodes.to_dict("index")

    # creating a network x graph from dataframes
    G = nx.from_pandas_edgelist(edge_data, edge_attr="weight")
    nx.set_node_attributes(G, node_attrs)

    # ...
    # graph = nx.from_pandas_edgelist(edge_data, edge_attr=True)
    # nx.set_node_attributes(graph, node_attrs)

    
    out_path = orcid_id + ".gexf"

    nx.write_gexf(G, out_path)



import itertools

import networkx as nx
import pandas as pd
from crossref.restful import Works
from pyvis.network import Network
from scholarly import scholarly

from rcn_py import orcid


def scholar_author_info(fullname):
    # Search author info by their name
    search_query_author = scholarly.search_author(fullname)
    author = scholarly.fill(next(search_query_author))
    # Get all the publication info of the author
    docs = []
    for i in range(len(author["publications"])):
        docs.append(author["publications"][i]["bib"])

    title = []
    coauthor_name = []
    abstract = []

    pub_df = pd.DataFrame()
    for doc in docs:
        try:
            search_pub = scholarly.search_pubs(doc["title"])
            pub_info = next(search_pub)
            title.append(pub_info["bib"]["title"])
            coauthor_name.append(pub_info["bib"]["author"])
            abstract.append(pub_info["bib"]["abstract"])
        except:
            continue
    pub_df["title"] = title
    pub_df["authors"] = coauthor_name
    pub_df["abstarct"] = abstract

    # pub_df.to_csv(folderpath + "/" + fullname + "_scholar.csv")
    return pub_df


def crossref_author_by_orcid(orcid_id):
    # Extract author info using ORCID
    orcid_record = orcid.query_orcid_for_record(orcid_id)
    if orcid_record is False:
        return
    docs = orcid.extract_works_section(orcid_record)

    # Extarct publication info from Crossref
    title_list = []
    doi_list = []
    coauthor_name = []
    coauthor_orcid = []
    abstract = []
    works = Works()
    for doc in docs:
        doi, title = orcid.extract_doi(doc)
        w = works.doi(doi)
        orcid_list, name_list = orcid.get_authors(doi)

        title_list.append(title)
        doi_list.append(doi)
        if w:
            if "abstract" in w.keys():
                abstract.append(w["abstract"])
            else:
                abstract.append("")
        else:
            abstract.append("")
        coauthor_name.append(name_list)
        coauthor_orcid.append(orcid_list)

    pub_df = pd.DataFrame()
    pub_df["title"] = title_list
    pub_df["doi"] = doi_list
    pub_df["authors"] = coauthor_name
    pub_df["abstarct"] = abstract
    pub_df["author_orcid"] = coauthor_orcid

    return pub_df


# Store all publication info of one person
def crossref_author_by_name(fullname):
    # Extract author info using ORCID
    orcid_id = orcid.name_to_orcid_id(fullname)
    orcid_record = orcid.query_orcid_for_record(orcid_id)
    if orcid_record is False:
        return
    docs = orcid.extract_works_section(orcid_record)

    # Extarct publication info from Crossref
    title_list = []
    doi_list = []
    coauthor_name = []
    coauthor_orcid = []
    abstract = []
    works = Works()
    for doc in docs:
        doi, title = orcid.extract_doi(doc)
        w = works.doi(doi)
        orcid_list, name_list = orcid.get_authors(doi)

        title_list.append(title)
        doi_list.append(doi)
        if w:
            if "abstract" in w.keys():
                abstract.append(w["abstract"])
            else:
                abstract.append("")
        else:
            abstract.append("")
        coauthor_name.append(name_list)
        coauthor_orcid.append(orcid_list)

    pub_df = pd.DataFrame()
    pub_df["title"] = title_list
    pub_df["doi"] = doi_list
    pub_df["authors"] = coauthor_name
    pub_df["abstarct"] = abstract
    pub_df["author_orcid"] = coauthor_orcid

    # pub_df.to_csv(folderpath + "/" + fullname + "_crossref.csv")
    return pub_df


# Get all publications of all coauthors of the author, and do de-duplication
def coauthor_data_crossref(orcid_id):
    df = crossref_author_by_orcid(orcid_id)
   
    authors_list = df["author_orcid"]
    frames = []
    frames.append(df)
    for coauthors_id in authors_list:
        # coauthors_id = coauthors_id.strip("[")
        # coauthors_id = coauthors_id.strip("]")
        # coauthors_id = coauthors_id.split(",")
        for id in coauthors_id:
            id = id.strip("' ")
            orcid_record = orcid.query_orcid_for_record(id)
            if orcid_record["person"]["addresses"]["address"]:
                country = orcid_record["person"]["addresses"]["address"][0]["country"]["value"]
            else:
                country = ""
            if country == "NL":
                if id != orcid_id:
                    df2 = crossref_author_by_orcid(id)
                    if not df2.empty:
                        frames.append(df2)
    new_df = pd.concat(frames)
    new_df2 = new_df.drop_duplicates(subset=["doi"], keep="first", ignore_index=True)
    # new_df2.to_csv(folderpath + "/" + fullname + "_coauthors_allpub.csv")
    return new_df2


# Read out all co-authorship relationships from stored publication data
def get_links_from_coauthor_rel(orcid_id):
    all_df = coauthor_data_crossref(orcid_id)
    authors_list = all_df["author_orcid"]
    links = []
    for coauthors_id in authors_list:
        # res = coauthors_id.strip("[")
        # res = res.strip("]")
        # res = res.split(",")

        if len(coauthors_id) >= 2: # Only choose the paper that has 2+ authors
            links = links + list(itertools.combinations(coauthors_id, 2)) # make coauthor pairs
    edge_data = pd.DataFrame()
    sources = []
    targets = []
    for link in links:
        sources.append(link[0].strip("' "))
        targets.append(link[1].strip("' "))
    edge_data["source"] = sources
    edge_data["target"] = targets
    # edge_data.to_csv(folderpath + "/" + fullname + "_coauthors_link.csv")
    return edge_data, links


# Group the authors in the stored data, 
# and generate the node dictionary that will be used to build the network
def assign_group_node(orcid_id):
    all_df = coauthor_data_crossref(orcid_id)
    dois = all_df["doi"]
    authors_orcid = all_df["author_orcid"]
    authors_name = all_df["authors"]

    clusters, idx2topics = orcid.orcid_lda_cluster(dois)
    node_data = pd.DataFrame()
    orcid_id = []
    name = []
    group = []
    for i in range(len(authors_orcid)):
        # authors_orcid[i] = authors_orcid[i].strip("[")
        # authors_orcid[i] = authors_orcid[i].strip("]")
        # authors_orcid[i] = authors_orcid[i].split(",")
        # authors_name[i] = authors_name[i].strip("[")
        # authors_name[i] = authors_name[i].strip("]")
        # authors_name[i] = authors_name[i].split(",")

        for j in range(len(authors_orcid[i])):
            temp_orcid = authors_orcid[i][j].strip("' ")
            temp_name = authors_name[i][j].strip("' ")
            orcid_id.append(temp_orcid)
            name.append(temp_name)
            group.append(clusters[dois[i]])

    node_data["orcid"] = orcid_id
    node_data["name"] = name
    node_data["group"] = group
    node_data["topics"] = idx2topics[group]

    new_node_data = node_data.drop_duplicates(
        subset=["orcid"], keep="first", ignore_index=True
    )

    return new_node_data

# Simple networkX & pyvis visualization
def build_networkx(orcid_id):
    node_data = assign_group_node(orcid_id)
    edge_data, link = get_links_from_coauthor_rel(orcid_id)
    weights = []
    for i in link:
        weights.append(link.count(i))
    edge_data["weight"] = weights

    # getting a group id
    groups = node_data.groupby("group")["orcid"].apply(list).reset_index()
    groups["group"] = groups.index

    # finding group id for each node from groups dataframe
    nodes = node_data.merge(groups, how="left", on=["group"])
    nodes["title"] = node_data[["name"]]
    nodes["label"] = node_data[["name"]]
    nodes = nodes.drop("orcid_y", axis=1).set_index("orcid_x")

    # Pyvis network
    N = Network(height=800, width="100%", bgcolor="#222222", font_color="white")
    N.toggle_hide_edges_on_drag(False)
    N.barnes_hut()

    # collecting node attributes for network x
    node_attrs = nodes.to_dict("index")

    # creating a network x graph from dataframes
    graph = nx.from_pandas_edgelist(edge_data, edge_attr=True)
    nx.set_node_attributes(graph, node_attrs)

    fullname = orcid.from_orcid_to_name(orcid_id)
    lastname = fullname.split(" ")[-1]
    N.from_nx(graph)
    N.show(lastname + "_datafile.html")





# Add constraint for Person nodes and Publication nodes
def add_constraint(tx):
    """
    This function is used to add constraints to the nodes in a Neo4j database. 
    It ensures that the 'doi' property of Publication nodes and the 'scopus_id' property of Person nodes are unique.
    """

    # This Cypher command creates a unique constraint on the 'doi' property of Publication nodes.
    # If a constraint by the name 'pub_doi' does not exist, it will be created.
    tx.run("""
            CREATE CONSTRAINT pub_doi IF NOT EXISTS
            FOR (p:Publication) REQUIRE p.doi IS UNIQUE
            """)
    # This Cypher command creates a unique constraint on the 'scopus_id' property of Person nodes.
    # If a constraint by the name 'scopus_id' does not exist, it will be created.
    tx.run("""
            CREATE CONSTRAINT scopus_id IF NOT EXISTS
            FOR (n:Person) REQUIRE n.scopus_id IS UNIQUE
            """)



def neo4j_create_people(tx, df, subject):
    """
    This function is used to create nodes for people in a Neo4j database. 
    It iterates over each row of the input DataFrame, parsing and adding necessary information.

    Parameters:
    tx (neo4j.Session): A session object for Neo4j to run Cypher commands.
    df (pandas.DataFrame): A DataFrame containing information about the people to be added.
    subject (str): The subject associated with the people being added.
    """

    # Iterate over each row in the DataFrame
    for i in range(len(df)):
        # Skip rows where 'Author(s) ID' is not a string
        if not isinstance(df['Author(s) ID'][i], str):
            continue
        author_scopus_id = df["Author(s) ID"][i].split(";")[0:-1]

        # Skip rows with less than two authors
        if len(author_scopus_id) < 2:
            continue
        year = int(df.Year[i])
        author_name = df["Authors"][i].split(", ")[0:len(author_scopus_id)]

        # Extract the author's affiliations and country
        author_aff = df['Authors with affiliations'][i].split("; ")[0:len(author_scopus_id)]
        if author_aff[-1] == ".":
            author_aff = ""
            author_country = ""
        else:
            author_country = [aff.split(", ")[-1] for aff in author_aff]
        
        # Extract the author's keywords
        if not isinstance(df['Author Keywords'][i], str):
            if not isinstance(df['Index Keywords'][i], str):
                keywords = []
            else: 
                index_key = df["Index Keywords"][i].split("; ")
                new_index = []
                for k in index_key:
                    if k.count(" ") < 1:
                        new_index.append(k)
                keywords = new_index
        else:
            keywords = df["Author Keywords"][i].split("; ")

        # Iterate over each author in the list of authors
        for n in range(len(author_scopus_id)):
            # If the person exists, append keywords and year
            # Avoid adding duplicate years
            
            # Use the MERGE command to either create a new node or update an existing one
            # Use the apoc.coll.toSet function from the APOC plugin to add new items to a list property without creating duplicates
            
            tx.run("""
                MERGE (p:Person {scopus_id: $id})
                SET p.name = $name,
                    p.affiliation = $affiliation, 
                    p.country = $country,
                    p.keywords = apoc.coll.toSet(coalesce(p.keywords, []) + $keywords),
                    p.year = apoc.coll.toSet(coalesce(p.year, []) + $year),
                    p.subject = apoc.coll.toSet(coalesce(p.subject, []) + $subject)
                """, 
                id = author_scopus_id[n],
                name = author_name[n],
                affiliation = author_aff[n],
                country = author_country[n],
                keywords = keywords,
                year = year,
                subject = subject
                )
        
        

def neo4j_create_publication(tx, df, subject):
    """
    This function is used to create nodes for publications in a Neo4j database. 
    It iterates over each row of the input DataFrame, parsing and adding necessary information.

    Parameters:
    tx (neo4j.Session): A session object for Neo4j to run Cypher commands.
    df (pandas.DataFrame): A DataFrame containing information about the publications to be added.
    subject (str): The subject associated with the publications being added.
    """

    # Iterate over each row in the DataFrame
    for i in range(len(df)):
        # Skip rows where 'Author(s) ID' is not a string
        if not isinstance(df['Author(s) ID'][i], str):
            continue
        author_scopus_id = df["Author(s) ID"][i].split(";")[0:-1]

        # Skip rows with less than two authors
        if len(author_scopus_id) < 2:
            continue

        # Extract necessary publication information
        doi = df.DOI[i]
        title = df.Title[i]
        year = int(df.Year[i])
        cited = df["Cited by"][i]
        
        # Extract the publication's keywords
        if not isinstance(df['Author Keywords'][i], str):
            if not isinstance(df['Index Keywords'][i], str):
                keywords = []
            else: 
                index_key = df["Index Keywords"][i].split("; ")
                new_index = []
                for k in index_key:
                    if k.count(" ") < 1:
                        new_index.append(k)
                keywords = new_index
        else:
            keywords = df["Author Keywords"][i].split("; ")

        # Use the MERGE command to either create a new node or update an existing one
        # Use the apoc.coll.toSet function from the APOC plugin to add new items to a list property without creating duplicates
        tx.run("""
                MERGE (p:Publication {doi: $doi})
                SET p.title = $title,
                    p.year = $year, 
                    p.cited = $cited,
                    p.keywords = $keywords,
                    p.subject = apoc.coll.toSet(coalesce(p.subject, []) + $subject)
                """, 
                doi = doi,
                title = title,
                year = year,
                cited = cited,
                keywords = keywords,
                subject = subject
                )
        
        
        
        
def neo4j_create_author_pub_edge(tx, df):
    """
    This function is used to create edges between authors and publications in a Neo4j database. 
    It iterates over each row of the input DataFrame, creating a relationship for each author-publication pair.

    Parameters:
    tx (neo4j.Session): A session object for Neo4j to run Cypher commands.
    df (pandas.DataFrame): A DataFrame containing information about the publications and authors to be added.
    """
    
    # Iterate over each row in the DataFrame
    for i in range(len(df)):
        # Skip rows where 'Author(s) ID' is not a string
        if not isinstance(df['Author(s) ID'][i], str):
            continue
        author_scopus_id = df["Author(s) ID"][i].split(";")[0:-1]
        # Skip rows with less than two authors
        if len(author_scopus_id) < 2:
            continue     

        # Extract necessary author-publication relationship information
        author_name = df["Authors"][i].split(", ")[0:len(author_scopus_id)]   
        year = int(df.Year[i])
        doi = df.DOI[i]
        title = df.Title[i]

        # For each author-publication pair, create an IS_AUTHOR_OF relationship
        for i in range(len(author_scopus_id)):
            tx.run("""
                    MATCH 
                        (n:Person {scopus_id: $person_id}), 
                        (p:Publication {doi: $doi})
                    MERGE (n)-[r:IS_AUTHOR_OF]->(p)
                    ON CREATE SET 
                        r.year = $year,
                        r.author_name = $author_name,
                        r.title = $title
                    """, 
                    person_id = author_scopus_id[i], 
                    doi = doi, 
                    year = year,
                    author_name = author_name[i],
                    title = title
                    )
            


# Input the scopus csv filepath, and its main subject
# and Neo4j driver uri, username and password

# def execution(filepath, subject, uri, user, password):
#     with GraphDatabase.driver(uri, auth=(user, password)) as driver:
#         driver.verify_connectivity()
#         with driver.session(database="neo4j") as session:
#             # Create nodes & edges
#             if os.path.exists(filepath):
#             # Skipping bad lines (very rare occurrence): 
#             # Replace the following line: df = pd.read_csv(path, on_bad_lines = 'skip')
#                 df = pd.read_csv(filepath)
                    
#                 session.execute_write(neo4j_create_people, df, subject) 
#                 session.execute_write(neo4j_create_publication, df, subject)
#                 session.execute_write(neo4j_create_author_pub_edge, df)
#                 print ("Successfully insert " + subject + " csv file.")  
#             else:
#                 print("Filepath doesn't exist!") 




import requests
import sqlite3
import sys
sys.path.append("..")
from rcn_py import orcid
import gensim
from itertools import combinations
from pyvis.network import Network
import networkx as nx

def create_rsd_database(db_path):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    # Create a project table
    cur.execute("CREATE TABLE project (id primary key, title, description)")

    # Create a project member table where the id is unique, 
    # the data includes author info and software id,
    # N-to-N mapping
    cur.execute("""
    CREATE TABLE author(
        id primary key, project, orcid, given_name, family_name, affiliation, 
        FOREIGN KEY(project) 
        REFERENCES project(id))
        """)
    
    # Create a software table
    cur.execute("CREATE TABLE software(id primary key, doi, brand_name, description)")

    # Create a software contributor table where the id is unique, 
    # the data includes contributor info and software id,
    # N-to-N mapping
    cur.execute("""
    CREATE TABLE contributor(
        id primary key, software, orcid, given_name, family_name, affiliation, 
        FOREIGN KEY(software) 
        REFERENCES software(id))
        """)
    
    con.commit()
    con.close()
    return "Created"


def store_rsd(db_path):
    response_project = requests.get(
        "https://research-software-directory.org/api/v1/project?select=id,title,description"
        )
    response_author = requests.get(
        "https://research-software-directory.org/api/v1/team_member?select=id,project,orcid,given_names,family_names, affiliation"
        )
    response_software = requests.get(
        "https://research-software-directory.org/api/v1/software?select=id,concept_doi,brand_name, description"
        )
    response_contributor = requests.get(
        "https://research-software-directory.org/api/v1/contributor?select=id,software,orcid,given_names,family_names,affiliation"
        )
    
    projects = response_project.json()
    authors_proj = response_author.json()
    software = response_software.json()
    contributor_soft = response_contributor.json()

    con = sqlite3.connect(db_path)
    cur = con.cursor()
    for p in projects:
        cur.execute("INSERT or IGNORE INTO project VALUES (?, ?, ?)", 
                    (p['id'], p['title'], p['description'])
            )
    
    for a in authors_proj:
        if a['orcid'] == None:
            a['orcid'] = orcid.name_to_orcid_id(a['given_names'] + ' ' + a['family_names'])
        cur.execute(
            "INSERT or IGNORE INTO author VALUES (?, ?, ?, ?, ?, ?)", 
            (a['id'], a['project'], a['orcid'], a['given_names'], a['family_names'], a['affiliation'])
            )
    
    for s in software:
        cur.execute("INSERT or IGNORE INTO software VALUES (?, ?, ?, ?)", 
                    (s['id'], s['concept_doi'], s['brand_name'], s['description']))
    
    for c in contributor_soft:
        if c['orcid'] == None:
            c['orcid'] = orcid.name_to_orcid_id(c['given_names'] + ' ' + c['family_names'])
        cur.execute("INSERT or IGNORE INTO contributor VALUES (?, ?, ?, ?, ?, ?)", 
                    (c['id'], c['software'], c['orcid'], c['given_names'], c['family_names'], c['affiliation']))
    con.commit()
    con.close()
    return "Done"



# Please choose a record_type: "project" or "software"
def project_software_cluster(db_path, record_type):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    if record_type == 'project':
        res = cur.execute("SELECT id, title, description FROM project")
    elif record_type == 'software':
        res = cur.execute("SELECT id, brand_name, description FROM software")
    else:
        return "Please choose the data you wish to access, project or software."
    id_description = res.fetchall()

    cleaned_corpus = []
    ids = []
    for pub in id_description:
        ids.append(pub[0])
        if pub[2] == None:
            cleaned_corpus.append(orcid.preprocess(pub[1]))
        else:
            cleaned_corpus.append(orcid.preprocess(pub[1]+' '+pub[2]))
    cur.close()
    
    dictionary = gensim.corpora.Dictionary(cleaned_corpus) 
    corpus = [dictionary.doc2bow(text) for text in cleaned_corpus]

    lda_model =  gensim.models.LdaMulticore(corpus, 
                                    num_topics = 8, 
                                    id2word = dictionary,                                    
                                    passes = 10,
                                    workers = 2)
        
    idx2topics = {}
    for idx, topic in lda_model.print_topics(-1):
        idx2topics[idx] = topic
    
    clusters = {}
    for i in range(len(cleaned_corpus)):
        scores = []
        topics = []
        for j1,j2 in lda_model[corpus[i]]:
            topics.append(j1)
            scores.append(j2)
            clusters[ids[i]] = scores.index(max(scores))

    return clusters, idx2topics


def author_cluster(db_path, type, clusters):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    if type == 'project':
        res = cur.execute("""
        SELECT orcid, GROUP_CONCAT(project,',') 
        FROM author 
        GROUP BY orcid
        """)
    elif type == 'software':
        res = cur.execute("""
        SELECT orcid, GROUP_CONCAT(software,',') 
        FROM contributor 
        GROUP BY orcid
        """)
    else:
        return "Please choose the data you wish to access, project or software."
    orc_id = res.fetchall()

    au_group = {}
    for au_pub in orc_id:
        pubs = au_pub[1].strip('\'')
        id_list = pubs.split(',')
        set_id_list = list(set(id_list))
        groups = []
        for id in set_id_list:
            groups.append(clusters[id])
        au_group[au_pub[0]] = max(groups,key=groups.count)
    cur.close()
    return au_group
    


def rsd_relationship(db_path, type):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    if type == 'project':
        res = cur.execute("""
        SELECT GROUP_CONCAT(orcid, ',') AS orcids 
        FROM author 
        GROUP BY project 
        HAVING COUNT(*) > 1
        """)
    elif type == 'software':
        res = cur.execute("""
        SELECT GROUP_CONCAT(orcid, ',') AS orcids 
        FROM contributor 
        GROUP BY software 
        HAVING COUNT(*) > 1
        """)
    else:
        return "Please choose the data you wish to access, project or software."
    coauthor = res.fetchall()

    coauthor_links = []
    for au in coauthor:
        au = au[0].strip('(\')')
        au = au.split(',')
        coauthor_links = coauthor_links+list(combinations(au, 2))
    cur.close()
    return coauthor_links


# pyvis
def build_network_pyvis(db_path, type):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    clusters, idx2topics = project_software_cluster(type)
    link = rsd_relationship(type)
    group = author_cluster(type, clusters)
    
    if type == 'project':
        res_authors = cur.execute("""
        SELECT orcid, given_name, family_name 
        FROM author 
        GROUP BY orcid
        """)
    elif type == 'software':
        res_authors = cur.execute("""
        SELECT orcid, given_name, family_name 
        FROM contributor 
        GROUP BY orcid
        """)
    else:
        return "Please choose the data you wish to access, project or software."
    all_authors = res_authors.fetchall()
    
    sources = []
    targets = []
    weights = []

    for i in link:
        sources.append(i[0])
        targets.append(i[1])
        weights.append(link.count(i))

    id2name = {}
    for author in all_authors:
        id2name[author[0]] = author[1]+' '+author[2]   
    
    # Pyvis network
    N = Network(height=800, width="100%", bgcolor="#222222", font_color="white")
    N.toggle_hide_edges_on_drag(False)
    N.barnes_hut()

    edge_data = zip(sources, targets, weights)

    for e in edge_data:
        src = e[0]
        dst = e[1]
        w = e[2]

        N.add_node(src, src, title=src, group=group[src])
        N.add_node(dst, dst, title=dst, group=group[dst])
        N.add_edge(src, dst, value=w)


    # add neighbor data to node hover data
    for node in N.nodes:
        node["title"] = idx2topics[group[node['id']]]
        node["label"] = id2name[node["id"]]

    cur.close()

    N.show(type+"_rsd.html")
    return "html file is created"



# gephi
def build_network_gephi(db_path, type):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    clusters, idx2topics = project_software_cluster(type)
    link = rsd_relationship(type)
    group = author_cluster(type, clusters)
    
    if type == 'project':
        res_authors = cur.execute("""
        SELECT orcid, given_name, family_name 
        FROM author 
        GROUP BY orcid
        """)
    elif type == 'software':
        res_authors = cur.execute("""
        SELECT orcid, given_name, family_name 
        FROM contributor 
        GROUP BY orcid
        """)
    else:
        return "Please choose the data you wish to access, project or software."
    all_authors = res_authors.fetchall()
    
    sources = []
    targets = []
    weights = []
    all_node_in_link = []

    for i in link:
        sources.append(i[0])
        targets.append(i[1])
        weights.append(link.count(i))
        all_node_in_link.append(i[0])
        all_node_in_link.append(i[1])

    id2name = {}
    for author in all_authors:
        id2name[author[0]] = author[1]+' '+author[2]   
    
    # networkx
    G = nx.Graph()

    edge_data = zip(sources, targets, weights)

    for e in edge_data:
        src = e[0]
        dst = e[1]
        w = e[2]

        G.add_node(src, group=group[src], title=idx2topics[group[src]], label=id2name[src], size=all_node_in_link.count(src))
        G.add_node(dst, group=group[dst], title=idx2topics[group[dst]], label=id2name[dst], size=all_node_in_link.count(dst))
        G.add_edge(src, dst, weight=w)

    cur.close()

    out_path =  type+ '_rsd.gexf'
   
    nx.write_gexf(G,out_path)
    return "gexf file is created"


# There are the SQLite DB functions
# Not used in the later development
# Was Replaced by Neo4j graph database

import sqlite3
from itertools import combinations

import gensim
from crossref.restful import Works
from pyvis.network import Network
from rcn_py import orcid

"""
    author-paper table
    author info table
    publication table
"""

def insert_database(db_path, orcid_id, fullname):
    if len(orcid_id) == 0:
        orcid_id = orcid.name_to_orcid_id(fullname)
    orcid_record = orcid.query_orcid_for_record(orcid_id)
    # probably there is null value in every stage, which will occure an error
    if len(orcid_record["person"]["addresses"]["address"]) > 0:
        country = orcid_record["person"]["addresses"]["address"][0]["country"]["value"]
    else:
        country = ""

    docs = orcid.extract_works_section(orcid_record)

    con = sqlite3.connect(db_path)
    cur = con.cursor()
    works = Works()
    orcid_doi_value = []
    paper_value = []
    for doc in docs:
        doi, title = orcid.extract_doi(doc)
        # orcid_list, name_list = orcid.get_authors(doi)
        # country_list = []
        # for orc in
        # make insert value for orcid-doi
        orcid_doi_value.append((orcid_id, doi))

        w = works.doi(doi)
        # orcid_list, name_list = orcid.get_authors(doi)

        if w:
            if "abstract" in w.keys():
                abstract = w["abstract"]
            else:
                abstract = ""
        else:
            abstract = ""

        paper_value.append((doi, title, abstract))

    # author table (orcid, name, country)
    cur.execute(
        "INSERT or IGNORE INTO authors VALUES (?, ?, ?)", (orcid_id, fullname, country)
    )
    # publication table (doi, title, abstract)
    cur.executemany("INSERT or IGNORE INTO publications VALUES (?, ?, ?)", paper_value)
    # author-paper table (orcid, doi)
    cur.executemany(
        "INSERT or IGNORE INTO author_publication VALUES (?, ?)", orcid_doi_value
    )
    con.commit()
    con.close()

    return "Done"


# Retrieve the coauthors for publications of the person
def insert_coauthors_pub(db_path, fullname):
    orcid_id = orcid.name_to_orcid_id(fullname)
    orcid_record = orcid.query_orcid_for_record(orcid_id)
    if orcid_record is False:
        return
    docs = orcid.extract_works_section(orcid_record)

    for doc in docs:
        doi, title = orcid.extract_doi(doc)
        orcid_list, name_list = orcid.get_authors(doi)
        for i in range(len(orcid_list)):
            insert_database(db_path, orcid_list[i], name_list[i])

    return "Done!"


def insert_cocoauthors(database):
    con = sqlite3.connect(database)
    cur = con.cursor()
    cur.execute("SELECT doi FROM publications")
    all_dois = cur.fetchall()
    for doi in all_dois[:100]:
        orcid_list, name_list = orcid.get_authors(doi[0])
        if len(orcid_list) <= 1:
            continue
        orcid_doi_value = []
        for i in range(len(orcid_list[:10])):
            orcid_record = orcid.query_orcid_for_record(orcid_list[i])
            # probably there is null value in every stage, which will occure an error
            if len(orcid_record["person"]["addresses"]["address"]) > 0:
                country = orcid_record["person"]["addresses"]["address"][0]["country"][
                    "value"
                ]
            else:
                country = ""
            orcid_doi_value.append((orcid_list[i], doi))
            cur.execute(
                "INSERT or IGNORE INTO authors VALUES (?, ?, ?)",
                (orcid_list[i], name_list[i], country),
            )

        # cur.executemany("INSERT or IGNORE INTO authors VALUES (?, ?, ?)", author_info)
        cur.executemany(
            "INSERT or IGNORE INTO author_publication VALUES (?, ?)", orcid_doi_value
        )

    con.commit()
    con.close()
    return "Done"


def pub_cluster(cur):
    res = cur.execute("SELECT doi, title, abstract FROM publications")
    all_pub = res.fetchall()
    cleaned_corpus = []
    dois = []
    for pub in all_pub:
        dois.append(pub[0])
        cleaned_corpus.append(orcid.preprocess(pub[1] + " " + pub[2]))
    dictionary = gensim.corpora.Dictionary(cleaned_corpus)
    corpus = [dictionary.doc2bow(text) for text in cleaned_corpus]

    lda_model = gensim.models.LdaMulticore(
        corpus, num_topics=8, id2word=dictionary, passes=10, workers=2
    )

    idx2topics = {}
    for idx, topic in lda_model.print_topics(-1):
        idx2topics[idx] = topic

    clusters = {}
    for i in range(len(cleaned_corpus)):
        scores = []
        topics = []
        for j1, j2 in lda_model[corpus[i]]:
            topics.append(j1)
            scores.append(j2)
            clusters[dois[i]] = scores.index(max(scores))
    return clusters, idx2topics


def fetch_relationships(cur):
    author_res = cur.execute("""
    SELECT GROUP_CONCAT(orcid, ',') AS orcids 
    FROM author_publication 
    WHERE doi != 'None' 
    GROUP BY doi 
    HAVING COUNT(*) > 1
    """)
    all_authors = author_res.fetchall()
    coauthor_links = []
    for au in all_authors:
        au = au[0].strip("(')")
        au = au.split(",")
        coauthor_links = coauthor_links + list(combinations(au, 2))
    return coauthor_links


def author_cluster(cur, clusters):
    author_res = cur.execute("""
    SELECT orcid
    GROUP_CONCAT(doi,',') 
    FROM author_publication 
    WHERE doi != 'None' 
    GROUP BY orcid
    """)
    
    orc_doi = author_res.fetchall()
    au_group = {}
    for au_pub in orc_doi:
        pubs = au_pub[1].strip("'")
        doi_list = pubs.split(",")
        groups = []
        for doi in doi_list:
            groups.append(clusters[doi])
        au_group[au_pub[0]] = max(groups, key=groups.count)
    return au_group


def build_network_database(db_path, name):
    con = sqlite3.connect(db_path)
    cur = con.cursor()
    clusters, idx2topics = pub_cluster(cur)
    link = fetch_relationships(cur)
    res_all_authors = cur.execute("SELECT orcid, name FROM authors")
    all_authors = res_all_authors.fetchall()
    group = author_cluster(cur, clusters)

    sources = []
    targets = []
    weights = []

    for i in link:
        sources.append(i[0])
        targets.append(i[1])
        weights.append(link.count(i))

    id2name = {}
    for author in all_authors:
        id2name[author[0]] = author[1]

    # Pyvis network
    N = Network(height=800, width="100%", bgcolor="#222222", font_color="white")
    N.toggle_hide_edges_on_drag(False)
    N.barnes_hut()

    edge_data = zip(sources, targets, weights)

    for e in edge_data:
        src = e[0]
        dst = e[1]
        w = e[2]

        N.add_node(src, src, title=src, group=group[src])
        N.add_node(dst, dst, title=dst, group=group[dst])
        N.add_edge(src, dst, value=w)

    # add neighbor data to node hover data
    for node in N.nodes:
        node["title"] = node["id"]
        node["label"] = cur.execute(
            "SELECT name FROM authors WHERE orcid == ?", (node["id"],)
        ).fetchall()[0][0]

    cur.close()

    N.show(name + ".html")



import itertools

import pandas as pd
from pyvis.network import Network
from scholarly import scholarly


# use scholarly package
def scholar_search_coauthor(author_fullname):
    # Search author info by their name
    search_query_author = scholarly.search_author(author_fullname)
    author = scholarly.fill(next(search_query_author))

    # Get all the publication info of the author
    docs = []
    for i in range(len(author["publications"])):
        docs.append(author["publications"][i]["bib"])

    # Filter by year
    # new_docs = []
    # for j in docs:
    #         new_docs.append(j)

    # Search the authors of each publication
    df = pd.DataFrame()
    link = []
    for doc in docs:
        try:
            search_pub = scholarly.search_pubs(doc["title"])
        except:
            continue
        else:
            pub_info = next(search_pub)
            # Build a dataframe containing authors' name and id
            temp_df = pd.DataFrame()
            temp_df["name"] = pub_info["bib"]["author"]
            temp_df["id"] = pub_info["author_id"]
        # Build links
        id_list = temp_df["id"]
        id_list_notnull = [i for i in id_list if i != ""]
        link = link + list(itertools.combinations(id_list_notnull, 2))

        # Join dfs
        frames = pd.concat([df, temp_df], ignore_index=True)
        df_notnull = frames[frames["id"] != ""]
        df = df_notnull.drop_duplicates(subset=["id"], keep="first", ignore_index=True)

    return df, link


def scholar_pyvis_network(fullname, node, link, au_group):
    sources = []
    targets = []
    weights = []

    for i in link:
        sources.append(i[0])
        targets.append(i[1])
        weights.append(link.count(i))

    # Pyvis network
    N = Network(height=800, width="100%", bgcolor="#222222", font_color="white")
    N.toggle_hide_edges_on_drag(False)
    N.barnes_hut()

    edge_data = zip(sources, targets, weights)

    for e in edge_data:
        src = e[0]
        dst = e[1]
        w = e[2]

        N.add_node(src, src, title=src, group=au_group[src])
        N.add_node(dst, dst, title=dst, group=au_group[dst])
        N.add_edge(src, dst, value=w)

    N.get_adj_list()

    # add neighbor data to node hover data
    for node in N.nodes:
        # node["title"] = " Neighbors: \n" + " \n".join(neighbors)
        node["title"] = (
            "Link to the author’s page:\n"
            + scholarly.search_author_id(node["id"])["url_picture"]
        )
        node["value"] = scholarly.search_author_id(node["id"])["citedby"]
        node["label"] = scholarly.search_author_id(node["id"])["name"]

    N.show(fullname + ".html")


def scholar_build_graph(G, sch_id, depth: int, bredth: int, last_author=[]):
    author_search = scholarly.search_author_id(sch_id)
    author = scholarly.fill(author_search, sections=["coauthors", "indices", "counts"])
    author_name = author["name"]
    author_hin = author["hindex"]
    author_ncite = author["citedby"]
    G.add_node(author_name, hindex=author_hin, name=author_name, ncite=author_ncite)
    print("now at %s, depth=%d" % (author_name, depth))
    if depth > 0:
        for coauthor in author["coauthors"][: int(bredth)]:
            G.add_edge(author_name, coauthor["name"])
            if coauthor["name"] not in last_author:
                last_author.append(author_name)
                scholar_build_graph(
                    G, coauthor["scholar_id"], depth - 1, bredth, last_author
                )
